<?php

namespace Payshop\Core\Action;

use Payshop\Core\Helper\PaymentMethods;
use Magento\Store\Model\StoreManagerInterface;
use Payshop\Core\Gateway\SDK\PayshopSDKClientFactory;
use Payshop\Core\Helper\PayshopConfigurationsComposite;

class CreateCharge
{
    /**
     * @var PaymentMethods
     */
    private PaymentMethods $paymentMethods;

    /**
     * @var StoreManagerInterface
     */
    private StoreManagerInterface $storeManager;

    /**
     * @var PayshopConfigurationsComposite
     */
    private PayshopConfigurationsComposite $config;

    /**
     * @var PayshopSDKClientFactory
     */
    private PayshopSDKClientFactory $payshopSDKClientFactory;

    /**
     * CreateCharge constructor.
     * 
     * @param PaymentMethods $paymentMethods
     * @param StoreManagerInterface $storeManager
     * @param PayshopConfigurationsComposite $config
     * @param PayshopSDKClientFactory $payshopSDKClientFactory
     */
    public function __construct(
        PaymentMethods $paymentMethods,
        StoreManagerInterface $storeManager,
        PayshopConfigurationsComposite $config,
        PayshopSDKClientFactory $payshopSDKClientFactory
    ) {
        $this->paymentMethods = $paymentMethods;
        $this->storeManager = $storeManager;
        $this->config = $config;
        $this->payshopSDKClientFactory = $payshopSDKClientFactory;
    }

    /**
     * Create charge on Payshop
     *
     * @param array $chargeData
     * @return array
     */
    public function execute(array $chargeData): array
    {
        $payshopClient = $this->payshopSDKClientFactory->getInstance();

        $chargeResponse = $payshopClient->createCharge($chargeData);
        $this->paymentMethods->checkResponse($chargeResponse);

        return $chargeResponse['response'];
    }

    /**
     * Get data to create charge
     *
     * @param object $data
     * @param string $paymentMethod
     * @return array
     */
    public function getData(object $data, string $paymentMethod): array
    {
        $paymentMethod = $this->paymentMethods->formatChargeType($paymentMethod);
        $storeName = $this->storeManager->getStore()->getName();

        return [
            'charge_type' => $paymentMethod,
            'amount' => $this->getAmount($data, $paymentMethod),
            'currency' => $this->getCurrency($data, $paymentMethod),
            'description' => __('Order from ') . $storeName,
            'events_url' => $this->webHookProcessEventURL(),
            'redirect_url' => $this->webhookProcessCardURL(),
            'instrument_params' => $this->getInstrumentParams($data, $paymentMethod) + [
                'description' => $storeName,
            ]
        ];
    }

    /**
     * Get currency code based on quote or order
     *
     * @param object $data
     * @param string $paymentMethod
     * @return string
     */
    private function getCurrency(object $data, string $paymentMethod): string
    {
        if ($paymentMethod === 'card') {
            return $data->getQuoteCurrencyCode();
        }

        return $data->getCurrencyCode();
    }

    /**
     * Get amount based on quote or order
     *
     * @param object $data
     * @param string $paymentMethod
     * @return float
     */
    private function getAmount(object $data, string $paymentMethod): float
    {
        if ($paymentMethod === 'card') {
            return $data->getTotals()['grand_total']->getValue();
        }

        return $data->getGrandTotalAmount();
    }

    /**
     * Get webhook process event url
     *
     * @return string
     */
    private function webHookProcessEventURL(): string
    {
        return $this->getStoreUrl() . 'payshop/webhook/process';
    }

    /**
     * Get webhook process event url
     *
     * @return string
     */
    private function webhookProcessCardURL(): string
    {
        return $this->getStoreUrl() . 'payshop/card/process';
    }

    /**
     * Get instrument params for the payment method
     *
     * @param object $data
     * @param string $paymentMethod
     * @return array
     */
    private function getInstrumentParams(object $data, string $paymentMethod): array
    {
        $paymentInTwoSteps = $this->config->getCard()->getValue(
            'payment_in_two_steps',
            $data->getStoreId()
        );

        $payshopReferenceExpirationDays = $this->config->getPayshopRefence()->getValue(
            'days_to_expire_reference',
            $data->getStoreId()
        );

        $multibancoReferenceExpirationDays = $this->config->getMultibancoReference()->getValue(
            'days_to_expire_reference',
            $data->getStoreId()
        );

        $paymentInTwoSteps = $paymentInTwoSteps ?? false;
        $payshopReferenceExpirationDays = $payshopReferenceExpirationDays ?? 2;
        $multibancoReferenceExpirationDays = $multibancoReferenceExpirationDays ?? 2;

        return $this->paymentMethods->getInstrumentParams(
            $paymentMethod,
            $paymentInTwoSteps,
            $payshopReferenceExpirationDays,
            $multibancoReferenceExpirationDays
        );
    }

    /**
     * Get store base url
     *
     * @return string
     */
    private function getStoreUrl(): string
    {
        /** @var \Magento\Framework\Url\ScopeInterface $store */
        $store = $this->storeManager->getStore();
        $storeUrl = $store->getBaseUrl();

        return $storeUrl == 'http://localhost/' ? 'https://eltonfonseca.dev/' : $storeUrl;
    }
}