<?php

namespace Payshop\Core\Controller\Webhook;

use Payshop\Core\Gateway\SDK\PayshopEvent;
use Magento\Framework\App\RequestInterface;
use Payshop\Core\Model\Events\ProcessWebhookEvent;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\Request\InvalidRequestException;

class Process implements HttpPostActionInterface, CsrfAwareActionInterface
{
    /**
     * @var JsonFactory
     */
    protected JsonFactory $resultJsonFactory;

    /**
     * @var ProcessWebhookEvent
     */
    private ProcessWebhookEvent $processWebhookEvent;

    /**
     * Process constructor.
     *
     * @param JsonFactory $resultJsonFactory
     * @param ProcessWebhookEvent $processWebhookEvent
     */
    public function __construct(
        JsonFactory $resultJsonFactory,
        ProcessWebhookEvent $processWebhookEvent
    ) {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->processWebhookEvent = $processWebhookEvent;
    }

    /**
     * Process the webhook request.
     * 
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultJsonFactory->create();

        try {
            $eventBasicInformation = PayshopEvent::getEvent();

            if ($this->unprocessableEvent($eventBasicInformation)) {
                return $response->setData(['status' => 'unprocessableEvent']);
            }

            $this->processWebhookEvent->execute($eventBasicInformation);

            return $response->setData(['status' => 'ok']);
        } catch (\Throwable $e) {
            return $response
                ->setHttpResponseCode(500)
                ->setData([
                    "status" => "error",
                    "message" => $e->getMessage()
                ]);
        }
    }

    /**
     * Check if the event is unprocessable
     *
     * @param array $eventBasicInformation
     * @return bool
     */
    private function unprocessableEvent(array $eventBasicInformation): bool
    {
        $eventType = $eventBasicInformation['event_type'];

        return !in_array($eventType, [
            'payment.success',
            'instrument.invalid'
        ]);
    }

    /**
     * Set the CSRF validation to true.
     * 
     * @param RequestInterface $request
     * @return bool|null
     */
    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }

    /**
     * Return null to avoid the CSRF validation.
     * 
     * @param RequestInterface $request
     * @return InvalidRequestException|null
     */
    public function createCsrfValidationException(
        RequestInterface $request
    ): ?InvalidRequestException
    {
        return null;
    }
}