<?php

namespace Payshop\Core\Helper;

class PaymentMethods
{
    /**
     * Get name of payment method
     * 
     * @param string $paymentMethod
     * @return string
     */
    public function formatChargeType(string $paymentMethod): string
    {
        switch ($paymentMethod) {
            case 'payshop_card':
                return 'card';
            case 'payshop_mbway':
                return 'mbway';
            case 'payshop_payshop_reference':
                return 'payshop_reference';
            case 'payshop_multibanco_reference':
                return 'multibanco';
        }
    }

    /**
     * Get instrument params for the payment method
     * 
     * @param string $paymentMethod
     * @param bool $paymentInTwoSteps
     * @param int $payshopReferenceExpirationDays
     * @param int $multibancoReferenceExpirationDays
     * @return array
     */
    public function getInstrumentParams(
        string $paymentMethod, 
        bool $paymentInTwoSteps,
        int $payshopReferenceExpirationDays,
        int $multibancoReferenceExpirationDays
    ): array {
        if ($paymentMethod == 'mbway') {
            return [];
        }

        if ($paymentMethod == 'card') {
            $params = ['enable3ds' => true];

            if ($paymentInTwoSteps) {
                $params['recurring'] = false;
                $params['funds_control'] = 'authorize';
            }

            return $params;
        }

        if ($paymentMethod == 'payshop_reference') {
            $qtdDaysToExpire = $payshopReferenceExpirationDays;
        }

        if ($paymentMethod == 'multibanco') {
            $qtdDaysToExpire = $multibancoReferenceExpirationDays;
        }

        $qtdDaysToExpire = $qtdDaysToExpire > 0 ? $qtdDaysToExpire : 2;

        return ['end_date' => date('Y-m-d', strtotime('+' . $qtdDaysToExpire . ' days'))];
    }

    /**
     * Check Payshop response
     *
     * @param array $response
     * @return bool|null
     * @throws Exception
     */
    public static function checkResponse(array $response): ?bool
    {
        if ($response['status'] == 201) {
            return true;
        }

        if ($response['status'] == 401 || $response['status'] == 403) {
            throw new \Exception('401403');
        }

        $body = $response['response'];

        if (isset($body['parameters']['events_url'][0])) {
            throw new \Exception('400001');
        }

        if (isset($body['parameters']['number'])) {
            throw new \Exception('100001');
        }

        if (isset($body['parameters']['phone'])) {
            throw new \Exception('100002');
        }

        $message = isset($body['message']) ? $body['message'] : $body;
        $code = $message == 'Transaction Error' ? '200001' : '200002';

        throw new \Exception($code);
    }
}